/* Sistemas Integrados TEKNISA

Nome: 520A_DFSSTPS_GERA_FECHAMENTO_NFENTRADA.SQL

Descrio:
Gera Fechamento de NF de Entrada

Criao:
04/10/2005 - Fabrcio Marcelo da Silva
*/

IF EXISTS (SELECT *
             FROM SYSOBJECTS O, SYSUSERS U
            WHERE
	           O.NAME = 'GERA_FECHAMENTO_NFENTRADA' AND
	           TYPE = 'P' 			  AND
              U.UID = O.UID        AND
              U.NAME = USER)

DROP PROCEDURE GERA_FECHAMENTO_NFENTRADA
GO

CREATE PROCEDURE GERA_FECHAMENTO_NFENTRADA
   @P_CDFILIAL     VARCHAR(04),
   @P_NRLANCTONF   VARCHAR(10),
   @P_DATAPROC     DATETIME    = NULL,
   @P_NRLANCESTQ   VARCHAR(10) = NULL,
   @P_CDOPERADOR   VARCHAR(12) = NULL
AS
   DECLARE @V_CDEMPRESA    VARCHAR(02);
   DECLARE @V_CDINSCESTA   VARCHAR(16);
   DECLARE @V_CDTIPOOPER   VARCHAR(02);
   DECLARE @V_CDCLIENTE    VARCHAR(21);
   DECLARE @V_CDFORNECED   VARCHAR(21);
   DECLARE @V_CDFILIRECE   VARCHAR(04);
   DECLARE @V_CLIENTENF    VARCHAR(21);
   DECLARE @V_ORIGDESTNF   VARCHAR(01);
   DECLARE @V_CDFILIEMIT   VARCHAR(04);
   DECLARE @V_IDGERAVENC   VARCHAR(01);
   DECLARE @V_IDUTILVENC   VARCHAR(01);

   DECLARE @V_DTINICIAL    DATETIME;
   DECLARE @V_DTFINAL      DATETIME;
   DECLARE @V_DTENTRSAID   DATETIME;
   DECLARE @V_VRNOTAFISC   DOUBLE PRECISION;
   DECLARE @V_VRLIQUIDO    DOUBLE PRECISION;
   DECLARE @V_DIFERENCA    DOUBLE PRECISION;

   DECLARE @V_VRIMPOSTOICST DOUBLE PRECISION;
   DECLARE @V_CDENCARGOST   VARCHAR(02);
   DECLARE @V_IDSTATUS   VARCHAR(01);
   DECLARE @V_IDPERLANESTAPNF VARCHAR(01);
BEGIN
   SET @V_CDEMPRESA    = NULL;
   SET @V_CDINSCESTA   = NULL;
   SET @V_CDTIPOOPER   = NULL;
   SET @V_CDCLIENTE    = NULL;
   SET @V_CDFORNECED   = NULL;
   SET @V_CDFILIRECE   = NULL;
   SET @V_DTENTRSAID   = NULL;
   SET @V_ORIGDESTNF   = NULL;
   SET @V_CDFILIEMIT   = NULL;
   SET @V_IDGERAVENC   = NULL;
   SET @V_IDUTILVENC   = NULL;
   SET @V_CDENCARGOST  = NULL;
   SET @V_VRLIQUIDO    = 0;
   SET @V_VRIMPOSTOICST = 0;

   SELECT @V_CDEMPRESA = MAX(N.CDEMPRESA), @V_CDINSCESTA = MAX(N.CDINSCESTA), @V_CDTIPOOPER = MAX(N.CDTIPOOPER),
          @V_CDCLIENTE = MAX(N.CDCLIENTE), @V_CDFORNECED = MAX(N.CDFORNECED), @V_CDFILIRECE = MAX(N.CDFILIRECE),
          @V_DTENTRSAID = MAX(N.DTENTRSAID), @V_ORIGDESTNF = MAX(N.IDORIGDESTNF), @V_CDFILIEMIT = MAX(N.CDFILIEMIT),
          @V_IDGERAVENC = ISNULL(MAX(T.IDGERAVENC), 'S'),
          @V_IDUTILVENC = ISNULL(MAX(T.IDUTILVENC), 'S')
   FROM NOTAFISCAL N, TIPOOPERAC T
   WHERE N.CDFILIAL     = @P_CDFILIAL     AND
         N.NRLANCTONF   = @P_NRLANCTONF   AND
         T.IDENTRSAIDOP = N.IDENTRSAIDOP  AND
         T.CDTIPOOPER   = N.CDTIPOOPER;

   IF @V_ORIGDESTNF = 'C' BEGIN
      SET @V_CLIENTENF = @V_CDCLIENTE;
   END
   ELSE IF @V_ORIGDESTNF = 'L' BEGIN
      SET @V_CLIENTENF = @V_CDFILIRECE;
   END
   ELSE BEGIN
      SET @V_CLIENTENF = @V_CDFORNECED;
   END;

   INSERT INTO ITEMENCARGO
      (CDFILIAL, NRLANCTONF, CDENCARGO, VRITEMENCA)
      (SELECT @P_CDFILIAL, @P_NRLANCTONF, CDENCARGO, CONVERT(FLOAT, 0)
       FROM ENCARGO
       WHERE CDENCARGO NOT IN (SELECT CDENCARGO
                               FROM ITEMENCARGO
                               WHERE CDFILIAL   = @P_CDFILIAL AND
                                     NRLANCTONF = @P_NRLANCTONF));

   SELECT @V_CDENCARGOST = IE.CDENCARGOST
     FROM NOTAFISCAL NF INNER JOIN INSCRESTAD IE ON NF.CDEMPRESA  = IE.CDEMPRESA
                                                AND NF.CDINSCESTA = IE.CDINSCESTA
    WHERE NF.CDFILIAL   = @P_CDFILIAL
      AND NF.NRLANCTONF = @P_NRLANCTONF
      AND IE.IDSUGREGSTENTR = 'N';

   IF @V_CDENCARGOST IS NOT NULL BEGIN
      SELECT @V_VRIMPOSTOICST = ISNULL(SUM(I.VRIMPOSTOIT),0)
        FROM ITEMPROIMPOS I INNER JOIN IMPOSTO M ON I.CDIMPOSTO = M.CDIMPOSTO
       WHERE I.CDFILIAL+I.NRLANCTONF = (SELECT DSNOTAFISORIGOL
                                           FROM NOTAFISCAL
                                          WHERE CDFILIAL   = @P_CDFILIAL
                                            AND NRLANCTONF = @P_NRLANCTONF)
         AND M.CDIMPOSTONFE = '80';

      UPDATE ITEMENCARGO
         SET VRITEMENCA = VRITEMENCA + @V_VRIMPOSTOICST
       WHERE CDFILIAL   = @P_CDFILIAL
         AND NRLANCTONF = @P_NRLANCTONF
         AND CDENCARGO  = @V_CDENCARGOST;
   END;

   EXEC TEKNISA.IMPOSTO_ENTRADA @V_CDEMPRESA, @V_CDINSCESTA, @V_CDTIPOOPER, @V_CLIENTENF,
                                @P_CDFILIAL,  @P_NRLANCTONF, @V_DTENTRSAID, @P_NRLANCESTQ,
                                'E',          @P_CDOPERADOR, NULL, @V_VRNOTAFISC OUTPUT;

   EXEC TEKNISA.GRAVA_IMPOSTO @P_CDFILIAL, @P_NRLANCTONF;

   IF (@V_IDGERAVENC = 'S') AND (@V_IDUTILVENC = 'S') BEGIN
      EXEC TEKNISA.GERA_CONTAPAGAR @P_CDFILIAL, @V_CLIENTENF, @P_NRLANCTONF, 'F';
   END;

   SELECT @V_VRLIQUIDO = SUM(VRTOTAITEM - VRDESCITEM)
   FROM ITEMPRODUTO
   WHERE CDFILIAL   = @P_CDFILIAL AND
         NRLANCTONF = @P_NRLANCTONF;

   UPDATE NOTAFISCAL
   SET VRNOTAFISC = @V_VRNOTAFISC,
       VRPRODUTOS = @V_VRLIQUIDO
   WHERE
       CDFILIAL   = @P_CDFILIAL AND
       NRLANCTONF = @P_NRLANCTONF;


   IF NOT @P_NRLANCESTQ IS NULL BEGIN

      SET @V_DIFERENCA = @V_VRNOTAFISC - @V_VRLIQUIDO;

      IF @V_VRIMPOSTOICST > 0 BEGIN
         EXEC TEKNISA.CUSTO_LIQUIDO @P_CDFILIAL, @P_NRLANCESTQ, @V_VRLIQUIDO, 0, 'N';
      END
      ELSE BEGIN
         EXEC TEKNISA.CUSTO_LIQUIDO @P_CDFILIAL, @P_NRLANCESTQ, @V_VRLIQUIDO, @V_DIFERENCA, 'N';
      END;

      IF @V_DTENTRSAID < @P_DATAPROC BEGIN
         SET @V_DTINICIAL = @V_DTENTRSAID;
      END
      ELSE BEGIN
         SET @V_DTINICIAL = @P_DATAPROC;
      END;

      IF @V_DTENTRSAID > @P_DATAPROC BEGIN
         SET @V_DTFINAL = @V_DTENTRSAID;
      END
      ELSE BEGIN
         SET @V_DTFINAL = @P_DATAPROC;
      END;

      SELECT @V_IDSTATUS = ISNULL(MAX(IDSTATUS), 'P')
        FROM NOTAFISCAL
       WHERE CDFILIAL   = @P_CDFILIAL
         AND NRLANCTONF = @P_NRLANCTONF;

      SELECT @V_IDPERLANESTAPNF = ISNULL(MAX(IDPERLANESTAPNF), 'N')
        FROM PARAMGERAL;


      IF (@V_IDSTATUS = 'I') OR (@V_IDSTATUS = 'L')  OR (@V_IDPERLANESTAPNF = 'S') BEGIN
         UPDATE ITLANCTOEST
            SET IDEVOLESTOQ  = 'N'
          WHERE CDFILIAL     = @V_CDFILIRECE
            AND NRLANCESTQ   = @P_NRLANCESTQ;
      END;

      EXEC TEKNISA.ATUALIZA_ESTOQUE @V_CDFILIRECE, @V_DTINICIAL, @V_DTFINAL, @P_CDFILIAL, @P_NRLANCESTQ;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'GERA_FECHAMENTO_NFENTRADA','6.35.028';
GO

